==> wow-test-3.4.6-sources.jar: me/ahoo/wow/test/saga/stateless/DefaultStatelessSagaVerifier.kt <==
/*
 * Copyright [2021-present] [ahoo wang <ahoowang@qq.com> (https://github.com/Ahoo-Wang)].
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *      http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package me.ahoo.wow.test.saga.stateless

import me.ahoo.wow.api.event.DomainEvent
import me.ahoo.wow.api.messaging.FunctionKind
import me.ahoo.wow.api.modeling.TenantId
import me.ahoo.wow.command.CommandGateway
import me.ahoo.wow.event.DomainEventExchange
import me.ahoo.wow.event.SimpleDomainEventExchange
import me.ahoo.wow.event.SimpleStateDomainEventExchange
import me.ahoo.wow.event.toDomainEvent
import me.ahoo.wow.id.GlobalIdGenerator
import me.ahoo.wow.infra.accessor.constructor.InjectableObjectFactory
import me.ahoo.wow.ioc.ServiceProvider
import me.ahoo.wow.messaging.processor.ProcessorMetadata
import me.ahoo.wow.saga.stateless.CommandStream
import me.ahoo.wow.saga.stateless.DefaultCommandStream
import me.ahoo.wow.saga.stateless.StatelessSagaFunctionRegistrar
import me.ahoo.wow.test.saga.stateless.GivenReadOnlyStateAggregate.Companion.toReadOnlyStateAggregate
import me.ahoo.wow.test.validation.validate
import reactor.core.publisher.Mono
import reactor.kotlin.core.publisher.switchIfEmpty
import reactor.kotlin.test.test
import java.lang.reflect.Constructor
import java.util.function.Consumer

internal class DefaultWhenStage<T : Any>(
    private val sagaMetadata: ProcessorMetadata<T, DomainEventExchange<*>>,
    private val serviceProvider: ServiceProvider,
    private val commandGateway: CommandGateway
) : WhenStage<T> {
    companion object {
        const val STATELESS_SAGA_COMMAND_ID = "__StatelessSagaVerifier__"
    }

    private fun toDomainEvent(event: Any): DomainEvent<*> {
        if (event is DomainEvent<*>) {
            return event
        }
        return event.toDomainEvent(
            aggregateId = GlobalIdGenerator.generateAsString(),
            tenantId = TenantId.DEFAULT_TENANT_ID,
            commandId = STATELESS_SAGA_COMMAND_ID,
        )
    }

    override fun <SERVICE : Any> inject(service: SERVICE, serviceName: String): WhenStage<T> {
        serviceProvider.register(serviceName, service)
        return this
    }

    @Suppress("UNCHECKED_CAST")
    override fun `when`(event: Any, state: Any?): ExpectStage<T> {
        val sagaCtor = sagaMetadata.processorType.constructors.first() as Constructor<T>
        val processor: T = InjectableObjectFactory(sagaCtor, serviceProvider).newInstance()
        val handlerRegistrar = StatelessSagaFunctionRegistrar(commandGateway)
        handlerRegistrar.registerProcessor(processor)
        val eventExchange = toEventExchange(event, state)
        val expectedResultMono = handlerRegistrar.supportedFunctions(eventExchange.message)
            .filter {
                if (state != null) {
                    it.functionKind == FunctionKind.STATE_EVENT
                } else {
                    it.functionKind == FunctionKind.EVENT
                }
            }
            .single()
            .invoke(eventExchange)
            .map {
                val commandStream = it as CommandStream
                commandStream.forEach { commandMessage ->
                    commandMessage.validate()
                }
                ExpectedResult(
                    processor = processor,
                    commandStream = it,
                )
            }
            .onErrorResume {
                Mono.just(
                    ExpectedResult(
                        processor = processor,
                        commandStream = null,
                        error = it
                    )
                )
            }.switchIfEmpty {
                Mono.just(
                    ExpectedResult(
                        processor = processor,
                        commandStream = DefaultCommandStream(
                            domainEventId = eventExchange.message.id,
                            commands = listOf()
                        )
                    )
                )
            }

        return DefaultExpectStage(expectedResultMono)
    }

    private fun toEventExchange(event: Any, state: Any?): DomainEventExchange<out Any> {
        val domainEvent = toDomainEvent(event)
        return if (state == null) {
            SimpleDomainEventExchange(message = domainEvent)
        } else {
            val stateAggregate = state.toReadOnlyStateAggregate(domainEvent)
            SimpleStateDomainEventExchange(state = stateAggregate, message = domainEvent)
        }.setServiceProvider(serviceProvider)
    }
}

internal class DefaultExpectStage<T : Any>(private val expectedResult: Mono<ExpectedResult<T>>) : ExpectStage<T> {
    private val expectStates: MutableList<Consumer<ExpectedResult<T>>> = mutableListOf()

    override fun expect(expected: Consumer<ExpectedResult<T>>): ExpectStage<T> {
        expectStates.add(expected)
        return this
    }

    override fun verify() {
        expectedResult
            .test()
            .consumeNextWith {
                for (expectState in expectStates) {
                    expectState.accept(it)
                }
            }
            .verifyComplete()
    }
}

==> wow-test-3.5.0-sources.jar: me/ahoo/wow/test/saga/stateless/DefaultStatelessSagaVerifier.kt <==
/*
 * Copyright [2021-present] [ahoo wang <ahoowang@qq.com> (https://github.com/Ahoo-Wang)].
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *      http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package me.ahoo.wow.test.saga.stateless

import me.ahoo.wow.api.event.DomainEvent
import me.ahoo.wow.api.messaging.FunctionKind
import me.ahoo.wow.api.modeling.TenantId
import me.ahoo.wow.command.CommandGateway
import me.ahoo.wow.command.factory.CommandMessageFactory
import me.ahoo.wow.event.DomainEventExchange
import me.ahoo.wow.event.SimpleDomainEventExchange
import me.ahoo.wow.event.SimpleStateDomainEventExchange
import me.ahoo.wow.event.toDomainEvent
import me.ahoo.wow.id.GlobalIdGenerator
import me.ahoo.wow.infra.accessor.constructor.InjectableObjectFactory
import me.ahoo.wow.ioc.ServiceProvider
import me.ahoo.wow.messaging.processor.ProcessorMetadata
import me.ahoo.wow.saga.stateless.CommandStream
import me.ahoo.wow.saga.stateless.DefaultCommandStream
import me.ahoo.wow.saga.stateless.StatelessSagaFunctionRegistrar
import me.ahoo.wow.test.saga.stateless.GivenReadOnlyStateAggregate.Companion.toReadOnlyStateAggregate
import me.ahoo.wow.test.validation.validate
import reactor.core.publisher.Mono
import reactor.kotlin.core.publisher.switchIfEmpty
import reactor.kotlin.test.test
import java.lang.reflect.Constructor
import java.util.function.Consumer

internal class DefaultWhenStage<T : Any>(
    private val sagaMetadata: ProcessorMetadata<T, DomainEventExchange<*>>,
    private val serviceProvider: ServiceProvider,
    private val commandGateway: CommandGateway,
    private val commandMessageFactory: CommandMessageFactory
) : WhenStage<T> {
    companion object {
        const val STATELESS_SAGA_COMMAND_ID = "__StatelessSagaVerifier__"
    }

    private fun toDomainEvent(event: Any): DomainEvent<*> {
        if (event is DomainEvent<*>) {
            return event
        }
        return event.toDomainEvent(
            aggregateId = GlobalIdGenerator.generateAsString(),
            tenantId = TenantId.DEFAULT_TENANT_ID,
            commandId = STATELESS_SAGA_COMMAND_ID,
        )
    }

    override fun <SERVICE : Any> inject(service: SERVICE, serviceName: String): WhenStage<T> {
        serviceProvider.register(serviceName, service)
        return this
    }

    @Suppress("UNCHECKED_CAST")
    override fun `when`(event: Any, state: Any?): ExpectStage<T> {
        val sagaCtor = sagaMetadata.processorType.constructors.first() as Constructor<T>
        val processor: T = InjectableObjectFactory(sagaCtor, serviceProvider).newInstance()
        val handlerRegistrar = StatelessSagaFunctionRegistrar(commandGateway, commandMessageFactory)
        handlerRegistrar.registerProcessor(processor)
        val eventExchange = toEventExchange(event, state)
        val expectedResultMono = handlerRegistrar.supportedFunctions(eventExchange.message)
            .filter {
                if (state != null) {
                    it.functionKind == FunctionKind.STATE_EVENT
                } else {
                    it.functionKind == FunctionKind.EVENT
                }
            }
            .single()
            .invoke(eventExchange)
            .map {
                val commandStream = it as CommandStream
                commandStream.forEach { commandMessage ->
                    commandMessage.validate()
                }
                ExpectedResult(
                    processor = processor,
                    commandStream = it,
                )
            }
            .onErrorResume {
                Mono.just(
                    ExpectedResult(
                        processor = processor,
                        commandStream = null,
                        error = it
                    )
                )
            }.switchIfEmpty {
                Mono.just(
                    ExpectedResult(
                        processor = processor,
                        commandStream = DefaultCommandStream(
                            domainEventId = eventExchange.message.id,
                            commands = listOf()
                        )
                    )
                )
            }

        return DefaultExpectStage(expectedResultMono)
    }

    private fun toEventExchange(event: Any, state: Any?): DomainEventExchange<out Any> {
        val domainEvent = toDomainEvent(event)
        return if (state == null) {
            SimpleDomainEventExchange(message = domainEvent)
        } else {
            val stateAggregate = state.toReadOnlyStateAggregate(domainEvent)
            SimpleStateDomainEventExchange(state = stateAggregate, message = domainEvent)
        }.setServiceProvider(serviceProvider)
    }
}

internal class DefaultExpectStage<T : Any>(private val expectedResult: Mono<ExpectedResult<T>>) : ExpectStage<T> {
    private val expectStates: MutableList<Consumer<ExpectedResult<T>>> = mutableListOf()

    override fun expect(expected: Consumer<ExpectedResult<T>>): ExpectStage<T> {
        expectStates.add(expected)
        return this
    }

    override fun verify() {
        expectedResult
            .test()
            .consumeNextWith {
                for (expectState in expectStates) {
                    expectState.accept(it)
                }
            }
            .verifyComplete()
    }
}
